#!/usr/bin/perl
#==============================================================================
# $Id: postcard-log.cgi,v 1.6 2000/05/12 21:51:24 peters Exp $
#
# Name:         postcard-log.cgi
#
# Written:      Peter Sundstrom (peters@ginini.com.au)
#
# Created:      Feb 1999
#
# Source:       http://www.ginini.com/software/postcard-direct/
#
# Description:  Displays the contents of the Postcard Direct log file.
#
# Copyright:    (c)1999-2000 Peter Sundstrom.
#
#		See http://www.ginini.com/software/postcard-direct/ for licence
#		details.
#
# Notes:	Please make sure that this script and log file is password
#		protected to make sure no unauthorised person can abuse the
#		information the log contains. 
#
#		If you can not do this, either disable logging or remove this
#		file.
#
# Version:	2.0.0
#
#==============================================================================

use CGI::Carp qw(carpout fatalsToBrowser);

require 5.004;

# Full directory path where the configuration file/s are kept
$ConfigDir="/home/ginini/htdocs/postcard-direct/config";

# Name of the default configuration filename if none is specified;
$DefaultCfg="$ConfigDir/default.cfg";

#------------------------------------------------------------------------
# END OF CONFIGURABLE OPTIONS
#------------------------------------------------------------------------

undef %Data;

$Method = $ENV{'REQUEST_METHOD'};

if ($Method eq 'GET') {
	$Query = $ENV{'QUERY_STRING'};
}
else {
	read(STDIN,$Query,$ENV{'CONTENT_LENGTH'});
}

foreach (split(/[&;]/, $Query)) {
	s/\+/ /g;
	($key, $value) = split('=', $_);
	$key =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;
	$value =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;
	$Data{$key} = $value;                
}

#
# Unbuffer output
#
$|=1;

#
# Check to see what configuration to use
#
$Config="$Data{config}";

if ($Config) {
	Error("Invalid Configuration file format") if ($Config !~ m#^([\w.-]+)$#);
        Error("Configuration file $ConfigDir/$Config does not exist") if ( ! -f "$ConfigDir/$Config");
        require "$ConfigDir/$Config";
}
else {
        Error("Default configuration file $DefaultCfg does not exist") if (! -f "$DefaultCfg");
        require "$DefaultCfg";
}

print <<HTML;
Content-type: text/html

<HTML>
<HEAD>
<META HTTP-EQUIV="PRAGMA" CONTENT="NO-CACHE">
<TITLE>Postcard Direct Log</TITLE>
</HEAD>

<BODY BGCOLOR="#FFFFFF">

<FONT FACE=Arial>
<CENTER>
<H1>Postcard Direct Log</H1>
</CENTER>
<P>


HTML

print "<A HREF=\"#Summary\">View Summary</A><P>\n" unless $LogSummary;

$Total=0;
$Top=10;

open (LOG,"$Logfile") or Error("Can not open log: $Logfile");

while (<LOG>) {
	if (m/\|/) {
		$Total++;
		@Field=split(/\|/);
		$Type{"$Field[2]"}++;
		$Design{"$Field[3]"}++;
		$Music{"$Field[4]"}++ if ($MUsic{"$Field[4]"});
		$Object{"$Field[5]"}++ if ($Object{"$Field[5]"});
		$Image{"$Field[6]"}++;
		$RemoteHost{"$Field[7]"}++;
		$Sender{"$Field[9]"}++;
		$Recipient{"$Field[11]"}++;;

		next if $LogSummary;

		print "<P><HR>\n";
		print "<TABLE BORDER=0>\n";
		print "<TR><TD><B>Date</B></TD><TD>$Field[0]</TD></TR>\n";
		print "<TR><TD><B>Time</B></TD><TD>$Field[1]</TD></TR>\n";
		print "<TR><TD><B>Type</B></TD><TD>$Field[2]</TD></TR>\n";
		print "<TR><TD><B>Design</B></TD><TD>$Field[3]</TD>\n";

		if ($Field[4] eq 'none') {
			print "<TR><TD><B>Music</B></TD><TD>$Field[4]</TD></TR>\n";
		}
		else {
			print "<TR><TD><B>Music</B></TD><TD><A HREF=$WebRoot$MidiURL/$Field[4]>$Field[4]</A></TD></TR>\n";
		}

		print "<TR><TD><B>Object</B></TD><TD>$Field[5]</TD></TR>\n" if $Field[5];
		print "<TR><TD><B>Image</B></TD><TD><A HREF=$WebRoot$Field[6]>$Field[6]</A></TD></TR>\n";
		print "<TR><TD><B>Remote Host</B></TD><TD>$Field[7]</TD></TR>\n";
		print "<TR><TD><B>Subject</B></TD><TD>$Field[8]</TD></TR>\n";
		print "<TR><TD><B>Sender Email</B></TD><TD>$Field[9]</TD></TR>\n";
		print "<TR><TD><B>Sender Name</B></TD><TD>$Field[10]</TD></TR>\n";
		print "<TR><TD><B>Recipient Email</B></TD><TD>$Field[11]</TD></TR>\n";
		print "<TR><TD><B>Recipient Name</B></TD><TD>$Field[12]</TD></TR>\n";
		print "</TABLE>\n";
		print "<BR>$Field[13]\n";
	}
	else {
		print "<BR>$_\n";
	}
}

close (LOG);

print <<HTML;

<P>
<HR>
<BR>
<A NAME="Summary"</A>
<H2>Postcard Direct Log Summary</H2>
<H3><B>Total Entries:</B> $Total</H3>

HTML

foreach $Column ('Type','Design','Music','Object','Image','RemoteHost','Sender', 'Recipient') {
	$Counter=0;

	undef @data;
	undef @datakeys;

	while (($Key,$Value) = each %$Column) {
                push(@data,"$Key $Value");
		push(@datakeys,$Value);
	}

	next unless @data;

	print "<TABLE BORDER=2 CELLSPACING=2 CELLPADDING=2>\n";
	print "<TR><TD WIDTH=%30><B>$Column</B></TD><TD WIDTH=10%><B>Entries</B></TD><TD><B>Percent</B></TD></TR>\n";

	@sortdata=@data[sort DataKeys_Numerically $[..$#data];

	foreach (@sortdata) {
		last if $Counter > $Top;
		($Entry,$Count)=/(.*) (\d+$)/;
		$Percent=($Count / $Total) * 100;
		print "<TR><TD>$Entry</TD><TD ALIGN=RIGHT>$Count</TD>";
		printf("<TD ALIGN=RIGHT>%2d%</TD>",$Percent);
		print "<TD WIDTH=200><HR NOSHADE ALIGN=LEFT SIZE=10 WIDTH=$Percent</TD></TR>\n";
		$Counter++;
	}

	print "</TABLE><P>\n";
		
}

print <<HTML;
</FONT>
</BODY>
</HTML>
HTML

#-----------------------------------------------------------------------------
sub DataKeys_Numerically {
        $datakeys[$b] <=> $datakeys[$a];
}

#-----------------------------------------------------------------------------
# Display any errors

sub Error {
	my $Text=shift;

	print "Content-type: text/html\n\n";
	print <<HTML;
<HTML>
<HEAD>
<TITLE>Postcard Direct Error</TITLE>
</HEAD>

<BODY BGCOLOR="#ffffff">

<P>
<FONT FACE=Arial SIZE=+1>
$Text
</FONT>
<P>

</BODY>
</HTML>
HTML

	exit;
}
